
#include <EmonLib_3PH.h>
#include <RFM12B_Due.h>

// You will need to initialize the radio by telling it what ID it has and what network it's on
// The NodeID takes values from 1-127, 0 is reserved for sending broadcast messages (send to all nodes)
// The Network ID takes values from 0-255
#define NODEID        20  //network ID used for this unit
#define NETWORKID    100  //the network ID we are on
#define GATEWAYID     10  //the node ID we're sending to
#define ACK_TIME      50  // # of ms to wait for an ack

// Need an instance of the Radio Module
RFM12B radio;


//==========================================================
#define NSENSORS 10

// Instance of EmonTX sensors
EnergyMonitor emon[NSENSORS];

// for each sensor we will send out real power and power factor
typedef struct {
  float realPower;    // 4
  float powerFactor;  // 4
} sEmonSensor;

// pack together data from 10 sensors, plus common voltage and time tick
typedef struct {
  unsigned long secTick;  // 4
  float voltage;          // 4
  sEmonSensor sensor[NSENSORS];  // 8 x N
} sEmonPayload;

// Instance of transmitted data structure
sEmonPayload EmonTx;
//==========================================================


// Time (ms) to allow the filters to settle before sending data
#define FILTERSETTLETIME 10000         
boolean settled = false;

unsigned long lastTick = 0;

const int LED = 13; //DUE


void setup() {
//  Serial.begin(57600);
  
  // DUE can have up to 12 bits of ADC resolution
  analogReadResolution(ADC_BITS);

  // Stove: brown (2)
  emon[0].voltage(11, 334.0, 2.0, 2);   // Voltage: pin, calib, phasecal, line phase.
  emon[0].current(0, 97.7);             // Current: pin, calib.
  // Stove: grey (3)
  emon[1].voltage(11, 334.0, 3.0, 3);   // Voltage: pin, calib, phasecal, line phase.
  emon[1].current(1, 96.2);             // Current: pin, calib.
  // Stove: black (1)
  emon[2].voltage(11, 334.0, 1.7, 1);   // Voltage: pin, calib, phasecal, line phase.
  emon[2].current(2, 95.8);             // Current: pin, calib.
  // Dishwasher (2)
  emon[3].voltage(11, 334.0, 2.0, 2);   // Voltage: pin, calib, phasecal, line phase.
  emon[3].current(3, 95.9);             // Current: pin, calib.
  // Kitchen (3)
  emon[4].voltage(11, 334.0, 3.0, 3);   // Voltage: pin, calib, phasecal, line phase.
  emon[4].current(4, 97.7);             // Current: pin, calib.
  // Living room (1)
  emon[5].voltage(11, 334.0, 1.7, 1);   // Voltage: pin, calib, phasecal, line phase.
  emon[5].current(5, 95.9);             // Current: pin, calib.
  // Common (2)
  emon[6].voltage(11, 334.0, 2.0, 2);   // Voltage: pin, calib, phasecal, line phase.
  emon[6].current(6, 96.4);             // Current: pin, calib.
  // Bath-Toilet (3)
  emon[7].voltage(11, 334.0, 3.0, 3);   // Voltage: pin, calib, phasecal, line phase.
  emon[7].current(7, 96.3);             // Current: pin, calib.
  // Washmachine (1)
  emon[8].voltage(11, 334.0, 1.7, 1);   // Voltage: pin, calib, phasecal, line phase.
  emon[8].current(8, 96.4);             // Current: pin, calib.
  // Floor light (1)
  emon[9].voltage(11, 334.0, 1.7, 1);   // Voltage: pin, calib, phasecal, line phase.
  emon[9].current(9, 95.7);             // Current: pin, calib.

  pinMode(LED, OUTPUT);
  digitalWrite(LED, 0); //OFF
  
  radio.Initialize(NODEID, RF12_868MHZ, NETWORKID);
}



void loop() {

  for (int i=0; i<NSENSORS; i++)
  {
    // perform measurements
    emon[i].calcVI(16,2000);    // 2 h.c. for buffering + 14 h.c. for measuring
    
    // convert and pack data
    EmonTx.sensor[i].realPower = (float)emon[i].realPower;
    EmonTx.sensor[i].powerFactor = (float)emon[i].powerFactor;
  }
  
  // Voltage sensor is connected to Washmachine line
  EmonTx.voltage = (float)emon[8].Vrms;

  // store count of seconds
  EmonTx.secTick = millis() / 1000;

  // because millis() returns to zero after 50 days ! 
  if (!settled && millis() > FILTERSETTLETIME) settled = true;

  // Transmit data out
  if (settled)
  {
    digitalWrite(LED, 1);

    // normally, loop time is about 1.3 sec,
    // so, let's wait for 2 sec interval to elapse before transmitting.
    // LED is a good indicator of wasted time here!
    while ( ((millis() - lastTick)%4096) < 2000 ) {}; 
    // remember this tick
    lastTick = millis();

    // start transmitting
    radio.Send(GATEWAYID, &EmonTx, sizeof(EmonTx), false);
    // wait until transmit finished
    radio.SendWait(0);
    
    digitalWrite(LED, 0);
  }
  else
  {
    lastTick = millis();
  }  

}
